import 'package:article_flutter_riverpod/widgets/space.dart';
import 'package:flutter/material.dart';
import 'package:flutter_riverpod/flutter_riverpod.dart';

final countServiceProvider = StateNotifierProvider<CountService, int>((ref) => CountService(42)); // <1>

class CountService extends StateNotifier<int> { // <2>
  CountService(int firstValue) : super(firstValue);

  void increment() { // <3>
    state++;
  }
}

class StatelessExample extends StatelessWidget {
  const StatelessExample({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return const ProviderScope(
      child: Counter(),
    );
  }
}

class Counter extends ConsumerWidget { // <4>
  const Counter({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context, WidgetRef ref) { // <5>
    final currentValue = ref.watch(countServiceProvider); // <6>

    return Column(
      mainAxisSize: MainAxisSize.min,
      children: [
        Text("Current value: $currentValue"), // <7>
        HSpace.m,
        ElevatedButton(
          onPressed: () => _onIncrement(ref), // <8>
          child: const Text("Increment"),
        ),
      ],
    );
  }

  void _onIncrement(WidgetRef ref) {
    ref.read(countServiceProvider.notifier).increment(); // <9>
  }
}
